"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutV1JobsJobIDRequestBodyTypedDict(TypedDict):
    r"""Update a job."""

    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field."""
    title: NotRequired[str]
    r"""The job title"""
    hire_date: NotRequired[str]
    r"""The date when the employee was hired or rehired for the job."""
    two_percent_shareholder: NotRequired[bool]
    r"""Whether the employee owns at least 2% of the company."""
    state_wc_covered: NotRequired[Nullable[bool]]
    r"""Whether this job is eligible for workers' compensation coverage in the state of Washington (WA)."""
    state_wc_class_code: NotRequired[Nullable[str]]
    r"""The risk class code for workers' compensation in Washington state. Please visit [Washington state's Risk Class page](https://www.lni.wa.gov/insurance/rates-risk-classes/risk-classes-for-workers-compensation/risk-class-lookup#/) to learn more."""


class PutV1JobsJobIDRequestBody(BaseModel):
    r"""Update a job."""

    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field."""

    title: Optional[str] = None
    r"""The job title"""

    hire_date: Optional[str] = None
    r"""The date when the employee was hired or rehired for the job."""

    two_percent_shareholder: Optional[bool] = None
    r"""Whether the employee owns at least 2% of the company."""

    state_wc_covered: OptionalNullable[bool] = UNSET
    r"""Whether this job is eligible for workers' compensation coverage in the state of Washington (WA)."""

    state_wc_class_code: OptionalNullable[str] = UNSET
    r"""The risk class code for workers' compensation in Washington state. Please visit [Washington state's Risk Class page](https://www.lni.wa.gov/insurance/rates-risk-classes/risk-classes-for-workers-compensation/risk-class-lookup#/) to learn more."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "title",
            "hire_date",
            "two_percent_shareholder",
            "state_wc_covered",
            "state_wc_class_code",
        ]
        nullable_fields = ["state_wc_covered", "state_wc_class_code"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PutV1JobsJobIDRequestTypedDict(TypedDict):
    job_id: str
    r"""The UUID of the job"""
    request_body: PutV1JobsJobIDRequestBodyTypedDict
    r"""Update a job."""
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1JobsJobIDRequest(BaseModel):
    job_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the job"""

    request_body: Annotated[
        PutV1JobsJobIDRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Update a job."""

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class I9AuthorizationDocumentTypedDict(TypedDict):
    r"""An employee's I-9 verification document"""

    uuid: str
    r"""The UUID of the I-9 verification document"""
    document_type: str
    r"""The document's document type"""
    document_title: str
    r"""The document's document title"""
    issuing_authority: str
    r"""The document's issuing authority"""
    expiration_date: NotRequired[str]
    r"""The document's expiration date"""


class I9AuthorizationDocument(BaseModel):
    r"""An employee's I-9 verification document"""

    uuid: str
    r"""The UUID of the I-9 verification document"""

    document_type: str
    r"""The document's document type"""

    document_title: str
    r"""The document's document title"""

    issuing_authority: str
    r"""The document's issuing authority"""

    expiration_date: Optional[str] = None
    r"""The document's expiration date"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gusto_embedded import models, utils
from gusto_embedded._hooks import HookContext
from gusto_embedded.types import OptionalNullable, UNSET
from gusto_embedded.utils import get_security_from_env
from typing import Mapping, Optional, Union


class HistoricalEmployees(BaseSDK):
    def update(
        self,
        *,
        company_uuid: str,
        historical_employee_uuid: str,
        version: str,
        first_name: str,
        last_name: str,
        date_of_birth: str,
        ssn: str,
        work_address: Union[
            models.PutV1HistoricalEmployeesWorkAddress,
            models.PutV1HistoricalEmployeesWorkAddressTypedDict,
        ],
        home_address: Union[
            models.PutV1HistoricalEmployeesHomeAddress,
            models.PutV1HistoricalEmployeesHomeAddressTypedDict,
        ],
        termination: Union[
            models.PutV1HistoricalEmployeesTermination,
            models.PutV1HistoricalEmployeesTerminationTypedDict,
        ],
        job: Union[
            models.PutV1HistoricalEmployeesJob,
            models.PutV1HistoricalEmployeesJobTypedDict,
        ],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        middle_initial: Optional[str] = None,
        preferred_first_name: Optional[str] = None,
        email: Optional[str] = None,
        employee_state_taxes: Optional[
            Union[
                models.PutV1HistoricalEmployeesEmployeeStateTaxes,
                models.PutV1HistoricalEmployeesEmployeeStateTaxesTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Employee:
        r"""Update a historical employee

        Update a historical employee, an employee that was previously dismissed from the company in the current year.

        scope: `employees:manage`

        :param company_uuid: The UUID of the company
        :param historical_employee_uuid: The UUID of the historical employee
        :param version: The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field.
        :param first_name:
        :param last_name:
        :param date_of_birth:
        :param ssn:
        :param work_address:
        :param home_address:
        :param termination:
        :param job:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param middle_initial:
        :param preferred_first_name:
        :param email: Optional. If provided, the email address will be saved to the employee.
        :param employee_state_taxes:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1HistoricalEmployeesRequest(
            company_uuid=company_uuid,
            historical_employee_uuid=historical_employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1HistoricalEmployeesRequestBody(
                version=version,
                first_name=first_name,
                middle_initial=middle_initial,
                last_name=last_name,
                preferred_first_name=preferred_first_name,
                date_of_birth=date_of_birth,
                ssn=ssn,
                work_address=utils.get_pydantic_model(
                    work_address, models.PutV1HistoricalEmployeesWorkAddress
                ),
                home_address=utils.get_pydantic_model(
                    home_address, models.PutV1HistoricalEmployeesHomeAddress
                ),
                termination=utils.get_pydantic_model(
                    termination, models.PutV1HistoricalEmployeesTermination
                ),
                email=email,
                job=utils.get_pydantic_model(job, models.PutV1HistoricalEmployeesJob),
                employee_state_taxes=utils.get_pydantic_model(
                    employee_state_taxes,
                    Optional[models.PutV1HistoricalEmployeesEmployeeStateTaxes],
                ),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/companies/{company_uuid}/historical_employees/{historical_employee_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1HistoricalEmployeesRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-historical_employees",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Employee)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_async(
        self,
        *,
        company_uuid: str,
        historical_employee_uuid: str,
        version: str,
        first_name: str,
        last_name: str,
        date_of_birth: str,
        ssn: str,
        work_address: Union[
            models.PutV1HistoricalEmployeesWorkAddress,
            models.PutV1HistoricalEmployeesWorkAddressTypedDict,
        ],
        home_address: Union[
            models.PutV1HistoricalEmployeesHomeAddress,
            models.PutV1HistoricalEmployeesHomeAddressTypedDict,
        ],
        termination: Union[
            models.PutV1HistoricalEmployeesTermination,
            models.PutV1HistoricalEmployeesTerminationTypedDict,
        ],
        job: Union[
            models.PutV1HistoricalEmployeesJob,
            models.PutV1HistoricalEmployeesJobTypedDict,
        ],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        middle_initial: Optional[str] = None,
        preferred_first_name: Optional[str] = None,
        email: Optional[str] = None,
        employee_state_taxes: Optional[
            Union[
                models.PutV1HistoricalEmployeesEmployeeStateTaxes,
                models.PutV1HistoricalEmployeesEmployeeStateTaxesTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Employee:
        r"""Update a historical employee

        Update a historical employee, an employee that was previously dismissed from the company in the current year.

        scope: `employees:manage`

        :param company_uuid: The UUID of the company
        :param historical_employee_uuid: The UUID of the historical employee
        :param version: The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field.
        :param first_name:
        :param last_name:
        :param date_of_birth:
        :param ssn:
        :param work_address:
        :param home_address:
        :param termination:
        :param job:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param middle_initial:
        :param preferred_first_name:
        :param email: Optional. If provided, the email address will be saved to the employee.
        :param employee_state_taxes:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1HistoricalEmployeesRequest(
            company_uuid=company_uuid,
            historical_employee_uuid=historical_employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1HistoricalEmployeesRequestBody(
                version=version,
                first_name=first_name,
                middle_initial=middle_initial,
                last_name=last_name,
                preferred_first_name=preferred_first_name,
                date_of_birth=date_of_birth,
                ssn=ssn,
                work_address=utils.get_pydantic_model(
                    work_address, models.PutV1HistoricalEmployeesWorkAddress
                ),
                home_address=utils.get_pydantic_model(
                    home_address, models.PutV1HistoricalEmployeesHomeAddress
                ),
                termination=utils.get_pydantic_model(
                    termination, models.PutV1HistoricalEmployeesTermination
                ),
                email=email,
                job=utils.get_pydantic_model(job, models.PutV1HistoricalEmployeesJob),
                employee_state_taxes=utils.get_pydantic_model(
                    employee_state_taxes,
                    Optional[models.PutV1HistoricalEmployeesEmployeeStateTaxes],
                ),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/companies/{company_uuid}/historical_employees/{historical_employee_uuid}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1HistoricalEmployeesRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-historical_employees",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Employee)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from enum import Enum
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutV1CompaniesCompanyIDPayrollsPaymentMethod(str, Enum):
    r"""The employee's compensation payment method. Invalid values will be ignored."""

    DIRECT_DEPOSIT = "Direct Deposit"
    CHECK = "Check"


class PutV1CompaniesCompanyIDPayrollsFixedCompensationsTypedDict(TypedDict):
    r"""An array of fixed compensations for the employee. Fixed compensations include tips, bonuses, and one time reimbursements."""

    name: NotRequired[str]
    r"""The name of the compensation. This also serves as the unique, immutable identifier for this compensation."""
    amount: NotRequired[str]
    r"""The amount of the compensation for the pay period."""
    job_uuid: NotRequired[str]
    r"""The UUID of the job for the compensation."""


class PutV1CompaniesCompanyIDPayrollsFixedCompensations(BaseModel):
    r"""An array of fixed compensations for the employee. Fixed compensations include tips, bonuses, and one time reimbursements."""

    name: Optional[str] = None
    r"""The name of the compensation. This also serves as the unique, immutable identifier for this compensation."""

    amount: Optional[str] = None
    r"""The amount of the compensation for the pay period."""

    job_uuid: Optional[str] = None
    r"""The UUID of the job for the compensation."""


class PutV1CompaniesCompanyIDPayrollsHourlyCompensationsTypedDict(TypedDict):
    r"""An array of hourly compensations for the employee. Hourly compensations include regular, overtime, and double overtime hours."""

    name: NotRequired[str]
    r"""The name of the compensation. This also serves as the unique, immutable identifier for this compensation."""
    hours: NotRequired[str]
    r"""The number of hours to be compensated for this pay period."""
    job_uuid: NotRequired[str]
    r"""The UUIDs of the job for the compensation."""


class PutV1CompaniesCompanyIDPayrollsHourlyCompensations(BaseModel):
    r"""An array of hourly compensations for the employee. Hourly compensations include regular, overtime, and double overtime hours."""

    name: Optional[str] = None
    r"""The name of the compensation. This also serves as the unique, immutable identifier for this compensation."""

    hours: Optional[str] = None
    r"""The number of hours to be compensated for this pay period."""

    job_uuid: Optional[str] = None
    r"""The UUIDs of the job for the compensation."""


class PutV1CompaniesCompanyIDPayrollsPaidTimeOffTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the PTO. This also serves as the unique, immutable identifier for the PTO. Must pass in name or policy_uuid but not both."""
    hours: NotRequired[str]
    r"""The hours of this PTO taken during the pay period."""
    policy_uuid: NotRequired[str]
    r"""The uuid of the PTO policy. Must pass in name or policy_uuid but not both."""
    final_payout_unused_hours_input: NotRequired[str]
    r"""The outstanding hours paid upon termination. This field is only applicable for termination payrolls."""


class PutV1CompaniesCompanyIDPayrollsPaidTimeOff(BaseModel):
    name: Optional[str] = None
    r"""The name of the PTO. This also serves as the unique, immutable identifier for the PTO. Must pass in name or policy_uuid but not both."""

    hours: Optional[str] = None
    r"""The hours of this PTO taken during the pay period."""

    policy_uuid: Optional[str] = None
    r"""The uuid of the PTO policy. Must pass in name or policy_uuid but not both."""

    final_payout_unused_hours_input: Optional[str] = None
    r"""The outstanding hours paid upon termination. This field is only applicable for termination payrolls."""


class PutV1CompaniesCompanyIDPayrollsEmployeeCompensationsTypedDict(TypedDict):
    employee_uuid: NotRequired[str]
    r"""The UUID of the employee."""
    version: NotRequired[str]
    r"""The current version of this employee compensation from the prepared payroll. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""
    excluded: NotRequired[bool]
    r"""This employee will be excluded from payroll calculation and will not be paid for the payroll."""
    payment_method: NotRequired[PutV1CompaniesCompanyIDPayrollsPaymentMethod]
    r"""The employee's compensation payment method. Invalid values will be ignored."""
    memo: NotRequired[str]
    r"""Custom text that will be printed as a personal note to the employee on a paystub."""
    fixed_compensations: NotRequired[
        List[PutV1CompaniesCompanyIDPayrollsFixedCompensationsTypedDict]
    ]
    hourly_compensations: NotRequired[
        List[PutV1CompaniesCompanyIDPayrollsHourlyCompensationsTypedDict]
    ]
    paid_time_off: NotRequired[
        List[PutV1CompaniesCompanyIDPayrollsPaidTimeOffTypedDict]
    ]
    r"""An array of all paid time off the employee is eligible for this pay period. Each paid time off object can be the name or the specific policy_uuid."""


class PutV1CompaniesCompanyIDPayrollsEmployeeCompensations(BaseModel):
    employee_uuid: Optional[str] = None
    r"""The UUID of the employee."""

    version: Optional[str] = None
    r"""The current version of this employee compensation from the prepared payroll. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""

    excluded: Optional[bool] = None
    r"""This employee will be excluded from payroll calculation and will not be paid for the payroll."""

    payment_method: Optional[PutV1CompaniesCompanyIDPayrollsPaymentMethod] = None
    r"""The employee's compensation payment method. Invalid values will be ignored."""

    memo: Optional[str] = None
    r"""Custom text that will be printed as a personal note to the employee on a paystub."""

    fixed_compensations: Optional[
        List[PutV1CompaniesCompanyIDPayrollsFixedCompensations]
    ] = None

    hourly_compensations: Optional[
        List[PutV1CompaniesCompanyIDPayrollsHourlyCompensations]
    ] = None

    paid_time_off: Optional[List[PutV1CompaniesCompanyIDPayrollsPaidTimeOff]] = None
    r"""An array of all paid time off the employee is eligible for this pay period. Each paid time off object can be the name or the specific policy_uuid."""


class PutV1CompaniesCompanyIDPayrollsRequestBodyTypedDict(TypedDict):
    employee_compensations: List[
        PutV1CompaniesCompanyIDPayrollsEmployeeCompensationsTypedDict
    ]


class PutV1CompaniesCompanyIDPayrollsRequestBody(BaseModel):
    employee_compensations: List[PutV1CompaniesCompanyIDPayrollsEmployeeCompensations]


class PutV1CompaniesCompanyIDPayrollsRequestTypedDict(TypedDict):
    company_id: str
    r"""The UUID of the company"""
    payroll_id: str
    r"""The UUID of the payroll"""
    request_body: PutV1CompaniesCompanyIDPayrollsRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1CompaniesCompanyIDPayrollsRequest(BaseModel):
    company_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    payroll_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the payroll"""

    request_body: Annotated[
        PutV1CompaniesCompanyIDPayrollsRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .earning_type import EarningType, EarningTypeTypedDict
from gusto_embedded.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class EarningTypeListTypedDict(TypedDict):
    r"""Example response"""

    default: NotRequired[List[EarningTypeTypedDict]]
    r"""The default earning types for the company."""
    custom: NotRequired[List[EarningTypeTypedDict]]
    r"""The custom earning types for the company."""


class EarningTypeList(BaseModel):
    r"""Example response"""

    default: Optional[List[EarningType]] = None
    r"""The default earning types for the company."""

    custom: Optional[List[EarningType]] = None
    r"""The custom earning types for the company."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .off_cycle_reason_type import OffCycleReasonType
from .payroll_credit_blockers_type import (
    PayrollCreditBlockersType,
    PayrollCreditBlockersTypeTypedDict,
)
from .payroll_pay_period_type import PayrollPayPeriodType, PayrollPayPeriodTypeTypedDict
from .payroll_payment_speed_changed_type import (
    PayrollPaymentSpeedChangedType,
    PayrollPaymentSpeedChangedTypeTypedDict,
)
from .payroll_payroll_status_meta_type import (
    PayrollPayrollStatusMetaType,
    PayrollPayrollStatusMetaTypeTypedDict,
)
from .payroll_submission_blockers_type import (
    PayrollSubmissionBlockersType,
    PayrollSubmissionBlockersTypeTypedDict,
)
from .payroll_totals_type import PayrollTotalsType, PayrollTotalsTypeTypedDict
from .payroll_withholding_pay_period_type import PayrollWithholdingPayPeriodType
from datetime import datetime
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class PayrollMinimalTypedDict(TypedDict):
    r"""Example response"""

    processed: bool
    r"""Whether or not the payroll has been successfully processed. Note that processed payrolls cannot be updated. Additionally, a payroll is not guaranteed to be processed just because the payroll deadline has passed. Late payrolls are not uncommon. Conversely, users may choose to run payroll before the payroll deadline."""
    uuid: str
    r"""The UUID of the payroll."""
    payroll_uuid: str
    r"""The UUID of the payroll."""
    company_uuid: str
    r"""The UUID of the company for the payroll."""
    payroll_deadline: NotRequired[datetime]
    r"""A timestamp that is the deadline for the payroll to be run in order for employees to be paid on time.  If payroll has not been run by the deadline, a prepare request will update both the check date and deadline to reflect the soonest employees can be paid and the deadline by which the payroll must be run in order for said check date to be met."""
    check_date: NotRequired[str]
    r"""The date on which employees will be paid for the payroll."""
    processed_date: NotRequired[str]
    r"""The date at which the payroll was processed. Null if the payroll isn't processed yet."""
    calculated_at: NotRequired[str]
    r"""A timestamp of the last valid payroll calculation. Null if there isn't a valid calculation."""
    off_cycle: NotRequired[bool]
    r"""Indicates whether the payroll is an off-cycle payroll"""
    off_cycle_reason: NotRequired[Nullable[OffCycleReasonType]]
    r"""The off-cycle reason. Only included for off-cycle payrolls."""
    auto_pilot: NotRequired[bool]
    r"""Indicates whether the payroll is an auto pilot payroll"""
    external: NotRequired[bool]
    r"""Indicates whether the payroll is an external payroll"""
    final_termination_payroll: NotRequired[bool]
    r"""Indicates whether the payroll is the final payroll for a terminated employee. Only included for off-cycle payrolls."""
    withholding_pay_period: NotRequired[PayrollWithholdingPayPeriodType]
    r"""The payment schedule tax rate the payroll is based on. Only included for off-cycle payrolls."""
    skip_regular_deductions: NotRequired[bool]
    r"""Block regular deductions and contributions for this payroll.  Only included for off-cycle payrolls."""
    fixed_withholding_rate: NotRequired[bool]
    r"""Enable taxes to be withheld at the IRS's required rate of 22% for federal income taxes. State income taxes will be taxed at the state's supplemental tax rate. Otherwise, we'll sum the entirety of the employee's wages and withhold taxes on the entire amount at the rate for regular wages. Only included for off-cycle payrolls."""
    pay_period: NotRequired[PayrollPayPeriodTypeTypedDict]
    payroll_status_meta: NotRequired[PayrollPayrollStatusMetaTypeTypedDict]
    r"""Information about the payroll's status and expected dates"""
    totals: NotRequired[PayrollTotalsTypeTypedDict]
    r"""The subtotals for the payroll."""
    payment_speed_changed: NotRequired[PayrollPaymentSpeedChangedTypeTypedDict]
    r"""Only applicable when a payroll is moved to four day processing instead of fast ach."""
    created_at: NotRequired[datetime]
    r"""Datetime for when the resource was created."""
    submission_blockers: NotRequired[List[PayrollSubmissionBlockersTypeTypedDict]]
    r"""Only included for processed or calculated payrolls"""
    credit_blockers: NotRequired[List[PayrollCreditBlockersTypeTypedDict]]
    r"""Only included for processed payrolls"""


class PayrollMinimal(BaseModel):
    r"""Example response"""

    processed: bool
    r"""Whether or not the payroll has been successfully processed. Note that processed payrolls cannot be updated. Additionally, a payroll is not guaranteed to be processed just because the payroll deadline has passed. Late payrolls are not uncommon. Conversely, users may choose to run payroll before the payroll deadline."""

    uuid: str
    r"""The UUID of the payroll."""

    payroll_uuid: str
    r"""The UUID of the payroll."""

    company_uuid: str
    r"""The UUID of the company for the payroll."""

    payroll_deadline: Optional[datetime] = None
    r"""A timestamp that is the deadline for the payroll to be run in order for employees to be paid on time.  If payroll has not been run by the deadline, a prepare request will update both the check date and deadline to reflect the soonest employees can be paid and the deadline by which the payroll must be run in order for said check date to be met."""

    check_date: Optional[str] = None
    r"""The date on which employees will be paid for the payroll."""

    processed_date: Optional[str] = None
    r"""The date at which the payroll was processed. Null if the payroll isn't processed yet."""

    calculated_at: Optional[str] = None
    r"""A timestamp of the last valid payroll calculation. Null if there isn't a valid calculation."""

    off_cycle: Optional[bool] = None
    r"""Indicates whether the payroll is an off-cycle payroll"""

    off_cycle_reason: OptionalNullable[OffCycleReasonType] = UNSET
    r"""The off-cycle reason. Only included for off-cycle payrolls."""

    auto_pilot: Optional[bool] = None
    r"""Indicates whether the payroll is an auto pilot payroll"""

    external: Optional[bool] = None
    r"""Indicates whether the payroll is an external payroll"""

    final_termination_payroll: Optional[bool] = None
    r"""Indicates whether the payroll is the final payroll for a terminated employee. Only included for off-cycle payrolls."""

    withholding_pay_period: Optional[PayrollWithholdingPayPeriodType] = None
    r"""The payment schedule tax rate the payroll is based on. Only included for off-cycle payrolls."""

    skip_regular_deductions: Optional[bool] = None
    r"""Block regular deductions and contributions for this payroll.  Only included for off-cycle payrolls."""

    fixed_withholding_rate: Optional[bool] = None
    r"""Enable taxes to be withheld at the IRS's required rate of 22% for federal income taxes. State income taxes will be taxed at the state's supplemental tax rate. Otherwise, we'll sum the entirety of the employee's wages and withhold taxes on the entire amount at the rate for regular wages. Only included for off-cycle payrolls."""

    pay_period: Optional[PayrollPayPeriodType] = None

    payroll_status_meta: Optional[PayrollPayrollStatusMetaType] = None
    r"""Information about the payroll's status and expected dates"""

    totals: Optional[PayrollTotalsType] = None
    r"""The subtotals for the payroll."""

    payment_speed_changed: Optional[PayrollPaymentSpeedChangedType] = None
    r"""Only applicable when a payroll is moved to four day processing instead of fast ach."""

    created_at: Optional[datetime] = None
    r"""Datetime for when the resource was created."""

    submission_blockers: Optional[List[PayrollSubmissionBlockersType]] = None
    r"""Only included for processed or calculated payrolls"""

    credit_blockers: Optional[List[PayrollCreditBlockersType]] = None
    r"""Only included for processed payrolls"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "payroll_deadline",
            "check_date",
            "processed_date",
            "calculated_at",
            "off_cycle",
            "off_cycle_reason",
            "auto_pilot",
            "external",
            "final_termination_payroll",
            "withholding_pay_period",
            "skip_regular_deductions",
            "fixed_withholding_rate",
            "pay_period",
            "payroll_status_meta",
            "totals",
            "payment_speed_changed",
            "created_at",
            "submission_blockers",
            "credit_blockers",
        ]
        nullable_fields = ["off_cycle_reason"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class GrossUpPayTypedDict(TypedDict):
    r"""Example response"""

    gross_up: NotRequired[str]
    r"""Gross up earnings."""


class GrossUpPay(BaseModel):
    r"""Example response"""

    gross_up: Optional[str] = None
    r"""Gross up earnings."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ActiveCompaniesTypedDict(TypedDict):
    company_uuid: NotRequired[str]
    r"""unique identifier for the company associated with the invoice data"""
    active_employees: NotRequired[int]
    r"""The number of active employees the company was or will be invoiced for that invoice period. Active employees are calculated as the count of onboarded employees hired before the end of the invoice period and not terminated before the start of the invoice period."""
    active_contractors: NotRequired[int]
    r"""The number of active contractors the company was or will be invoiced for that invoice period. Active contractors are calculated as any contractor with an active contractor payment during the invoice period."""
    initial_invoice_period: NotRequired[str]
    r"""The first invoice period for the company. This will either be the invoice period of the first invoice-able event (first payroll or contractor payment) or the date they migrated to embedded, whichever is later."""


class ActiveCompanies(BaseModel):
    company_uuid: Optional[str] = None
    r"""unique identifier for the company associated with the invoice data"""

    active_employees: Optional[int] = None
    r"""The number of active employees the company was or will be invoiced for that invoice period. Active employees are calculated as the count of onboarded employees hired before the end of the invoice period and not terminated before the start of the invoice period."""

    active_contractors: Optional[int] = None
    r"""The number of active contractors the company was or will be invoiced for that invoice period. Active contractors are calculated as any contractor with an active contractor payment during the invoice period."""

    initial_invoice_period: Optional[str] = None
    r"""The first invoice period for the company. This will either be the invoice period of the first invoice-able event (first payroll or contractor payment) or the date they migrated to embedded, whichever is later."""


class InvoiceDataTypedDict(TypedDict):
    r"""Representation of a partners invoice data"""

    active_companies: NotRequired[List[ActiveCompaniesTypedDict]]
    r"""The list of companies that are active within the invoice period"""


class InvoiceData(BaseModel):
    r"""Representation of a partners invoice data"""

    active_companies: Optional[List[ActiveCompanies]] = None
    r"""The list of companies that are active within the invoice period"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from datetime import date
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutV1HistoricalEmployeesWorkAddressTypedDict(TypedDict):
    location_uuid: NotRequired[str]
    r"""Reference to a company location"""


class PutV1HistoricalEmployeesWorkAddress(BaseModel):
    location_uuid: Optional[str] = None
    r"""Reference to a company location"""


class PutV1HistoricalEmployeesHomeAddressTypedDict(TypedDict):
    street_1: str
    city: str
    state: str
    zip: str
    street_2: NotRequired[Nullable[str]]


class PutV1HistoricalEmployeesHomeAddress(BaseModel):
    street_1: str

    city: str

    state: str

    zip: str

    street_2: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["street_2"]
        nullable_fields = ["street_2"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PutV1HistoricalEmployeesTerminationTypedDict(TypedDict):
    effective_date: NotRequired[date]
    r"""Date the employee was terminated from the company"""


class PutV1HistoricalEmployeesTermination(BaseModel):
    effective_date: Optional[date] = None
    r"""Date the employee was terminated from the company"""


class PutV1HistoricalEmployeesJobTypedDict(TypedDict):
    hire_date: NotRequired[date]
    r"""The date when the employee was hired to the company"""


class PutV1HistoricalEmployeesJob(BaseModel):
    hire_date: Optional[date] = None
    r"""The date when the employee was hired to the company"""


class PutV1HistoricalEmployeesEmployeeStateTaxesTypedDict(TypedDict):
    wc_covered: NotRequired[bool]
    r"""Whether this job is eligible for workers' compensation coverage in the states of Washington (WA) or Wyoming (WY)."""
    wc_class_code: NotRequired[str]
    r"""The risk class code for workers' compensation in Washington or Wyoming state. For Washington, visit [Washington state's Risk Class page](https://www.lni.wa.gov/insurance/rates-risk-classes/risk-classes-for-workers-compensation/risk-class-lookup#/) to learn more. For Wyoming you can search for the code online using [WY Workforce Services website](https://dws.wyo.gov/dws-division/workers-compensation/) or call the agency at (307) 235-3217."""


class PutV1HistoricalEmployeesEmployeeStateTaxes(BaseModel):
    wc_covered: Optional[bool] = None
    r"""Whether this job is eligible for workers' compensation coverage in the states of Washington (WA) or Wyoming (WY)."""

    wc_class_code: Optional[str] = None
    r"""The risk class code for workers' compensation in Washington or Wyoming state. For Washington, visit [Washington state's Risk Class page](https://www.lni.wa.gov/insurance/rates-risk-classes/risk-classes-for-workers-compensation/risk-class-lookup#/) to learn more. For Wyoming you can search for the code online using [WY Workforce Services website](https://dws.wyo.gov/dws-division/workers-compensation/) or call the agency at (307) 235-3217."""


class PutV1HistoricalEmployeesRequestBodyTypedDict(TypedDict):
    r"""Update a historical employee."""

    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""
    first_name: str
    last_name: str
    date_of_birth: str
    ssn: str
    work_address: PutV1HistoricalEmployeesWorkAddressTypedDict
    home_address: PutV1HistoricalEmployeesHomeAddressTypedDict
    termination: PutV1HistoricalEmployeesTerminationTypedDict
    job: PutV1HistoricalEmployeesJobTypedDict
    middle_initial: NotRequired[str]
    preferred_first_name: NotRequired[str]
    email: NotRequired[str]
    r"""Optional. If provided, the email address will be saved to the employee."""
    employee_state_taxes: NotRequired[
        PutV1HistoricalEmployeesEmployeeStateTaxesTypedDict
    ]


class PutV1HistoricalEmployeesRequestBody(BaseModel):
    r"""Update a historical employee."""

    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""

    first_name: str

    last_name: str

    date_of_birth: str

    ssn: str

    work_address: PutV1HistoricalEmployeesWorkAddress

    home_address: PutV1HistoricalEmployeesHomeAddress

    termination: PutV1HistoricalEmployeesTermination

    job: PutV1HistoricalEmployeesJob

    middle_initial: Optional[str] = None

    preferred_first_name: Optional[str] = None

    email: Optional[str] = None
    r"""Optional. If provided, the email address will be saved to the employee."""

    employee_state_taxes: Optional[PutV1HistoricalEmployeesEmployeeStateTaxes] = None


class PutV1HistoricalEmployeesRequestTypedDict(TypedDict):
    company_uuid: str
    r"""The UUID of the company"""
    historical_employee_uuid: str
    r"""The UUID of the historical employee"""
    request_body: PutV1HistoricalEmployeesRequestBodyTypedDict
    r"""Update a historical employee."""
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1HistoricalEmployeesRequest(BaseModel):
    company_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    historical_employee_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the historical employee"""

    request_body: Annotated[
        PutV1HistoricalEmployeesRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Update a historical employee."""

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PostCompaniesCompanyUUIDHolidayPayPolicyNewYearsDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyNewYearsDay(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyMlkDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyMlkDay(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyPresidentsDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyPresidentsDay(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyMemorialDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyMemorialDay(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyJuneteenthTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyJuneteenth(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyIndependenceDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyIndependenceDay(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyLaborDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyLaborDay(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyColumbusDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyColumbusDay(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyVeteransDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyVeteransDay(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyThanksgivingTypedDict(TypedDict):
    selected: NotRequired[bool]


class PostCompaniesCompanyUUIDHolidayPayPolicyThanksgiving(BaseModel):
    selected: Optional[bool] = None


class ChristmasDayTypedDict(TypedDict):
    selected: NotRequired[bool]


class ChristmasDay(BaseModel):
    selected: Optional[bool] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyFederalHolidaysTypedDict(TypedDict):
    r"""An object containing federal holiday objects, each containing a boolean selected property."""

    new_years_day: NotRequired[
        PostCompaniesCompanyUUIDHolidayPayPolicyNewYearsDayTypedDict
    ]
    mlk_day: NotRequired[PostCompaniesCompanyUUIDHolidayPayPolicyMlkDayTypedDict]
    presidents_day: NotRequired[
        PostCompaniesCompanyUUIDHolidayPayPolicyPresidentsDayTypedDict
    ]
    memorial_day: NotRequired[
        PostCompaniesCompanyUUIDHolidayPayPolicyMemorialDayTypedDict
    ]
    juneteenth: NotRequired[PostCompaniesCompanyUUIDHolidayPayPolicyJuneteenthTypedDict]
    independence_day: NotRequired[
        PostCompaniesCompanyUUIDHolidayPayPolicyIndependenceDayTypedDict
    ]
    labor_day: NotRequired[PostCompaniesCompanyUUIDHolidayPayPolicyLaborDayTypedDict]
    columbus_day: NotRequired[
        PostCompaniesCompanyUUIDHolidayPayPolicyColumbusDayTypedDict
    ]
    veterans_day: NotRequired[
        PostCompaniesCompanyUUIDHolidayPayPolicyVeteransDayTypedDict
    ]
    thanksgiving: NotRequired[
        PostCompaniesCompanyUUIDHolidayPayPolicyThanksgivingTypedDict
    ]
    christmas_day: NotRequired[ChristmasDayTypedDict]


class PostCompaniesCompanyUUIDHolidayPayPolicyFederalHolidays(BaseModel):
    r"""An object containing federal holiday objects, each containing a boolean selected property."""

    new_years_day: Optional[PostCompaniesCompanyUUIDHolidayPayPolicyNewYearsDay] = None

    mlk_day: Optional[PostCompaniesCompanyUUIDHolidayPayPolicyMlkDay] = None

    presidents_day: Optional[PostCompaniesCompanyUUIDHolidayPayPolicyPresidentsDay] = (
        None
    )

    memorial_day: Optional[PostCompaniesCompanyUUIDHolidayPayPolicyMemorialDay] = None

    juneteenth: Optional[PostCompaniesCompanyUUIDHolidayPayPolicyJuneteenth] = None

    independence_day: Optional[
        PostCompaniesCompanyUUIDHolidayPayPolicyIndependenceDay
    ] = None

    labor_day: Optional[PostCompaniesCompanyUUIDHolidayPayPolicyLaborDay] = None

    columbus_day: Optional[PostCompaniesCompanyUUIDHolidayPayPolicyColumbusDay] = None

    veterans_day: Optional[PostCompaniesCompanyUUIDHolidayPayPolicyVeteransDay] = None

    thanksgiving: Optional[PostCompaniesCompanyUUIDHolidayPayPolicyThanksgiving] = None

    christmas_day: Optional[ChristmasDay] = None


class PostCompaniesCompanyUUIDHolidayPayPolicyRequestBodyTypedDict(TypedDict):
    federal_holidays: NotRequired[
        PostCompaniesCompanyUUIDHolidayPayPolicyFederalHolidaysTypedDict
    ]
    r"""An object containing federal holiday objects, each containing a boolean selected property."""


class PostCompaniesCompanyUUIDHolidayPayPolicyRequestBody(BaseModel):
    federal_holidays: Optional[
        PostCompaniesCompanyUUIDHolidayPayPolicyFederalHolidays
    ] = None
    r"""An object containing federal holiday objects, each containing a boolean selected property."""


class PostCompaniesCompanyUUIDHolidayPayPolicyRequestTypedDict(TypedDict):
    company_uuid: str
    r"""The UUID of the company"""
    request_body: PostCompaniesCompanyUUIDHolidayPayPolicyRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PostCompaniesCompanyUUIDHolidayPayPolicyRequest(BaseModel):
    company_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    request_body: Annotated[
        PostCompaniesCompanyUUIDHolidayPayPolicyRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import date
from gusto_embedded.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ContractorPaymentReceiptTotalsTypedDict(TypedDict):
    r"""The subtotals for the contractor payment."""

    company_debit: NotRequired[str]
    r"""The total company debit for the contractor payment."""


class ContractorPaymentReceiptTotals(BaseModel):
    r"""The subtotals for the contractor payment."""

    company_debit: Optional[str] = None
    r"""The total company debit for the contractor payment."""


class ContractorPaymentsModelTypedDict(TypedDict):
    contractor_uuid: NotRequired[str]
    r"""The UUID of the contractor."""
    contractor_first_name: NotRequired[str]
    r"""The first name of the contractor. Applies when `contractor_type` is `Individual`."""
    contractor_last_name: NotRequired[str]
    r"""The last name of the contractor.  Applies when `contractor_type` is `Individual`."""
    contractor_business_name: NotRequired[str]
    r"""The business name of the contractor. Applies when `contractor_type` is `Business`."""
    contractor_type: NotRequired[str]
    r"""The type of contractor.

    `Individual` `Business`
    """
    payment_method: NotRequired[str]
    r"""The payment method.

    `Direct Deposit` `Check` `Historical Payment` `Correction Payment`
    """
    wage: NotRequired[str]
    r"""The fixed wage of the payment, regardless of hours worked."""
    bonus: NotRequired[str]
    r"""The bonus amount in the payment."""
    reimbursement: NotRequired[str]
    r"""The reimbursement amount in the payment."""


class ContractorPaymentsModel(BaseModel):
    contractor_uuid: Optional[str] = None
    r"""The UUID of the contractor."""

    contractor_first_name: Optional[str] = None
    r"""The first name of the contractor. Applies when `contractor_type` is `Individual`."""

    contractor_last_name: Optional[str] = None
    r"""The last name of the contractor.  Applies when `contractor_type` is `Individual`."""

    contractor_business_name: Optional[str] = None
    r"""The business name of the contractor. Applies when `contractor_type` is `Business`."""

    contractor_type: Optional[str] = None
    r"""The type of contractor.

    `Individual` `Business`
    """

    payment_method: Optional[str] = None
    r"""The payment method.

    `Direct Deposit` `Check` `Historical Payment` `Correction Payment`
    """

    wage: Optional[str] = None
    r"""The fixed wage of the payment, regardless of hours worked."""

    bonus: Optional[str] = None
    r"""The bonus amount in the payment."""

    reimbursement: Optional[str] = None
    r"""The reimbursement amount in the payment."""


class ContractorPaymentReceiptLicenseeTypedDict(TypedDict):
    r"""The licensed payroll processor"""

    name: NotRequired[str]
    r"""Always the fixed string \"Gusto, Zenpayroll Inc.\" """
    address: NotRequired[str]
    r"""Always the fixed string \"525 20th St\" """
    city: NotRequired[str]
    r"""Always the fixed string \"San Francisco\" """
    state: NotRequired[str]
    r"""Always the fixed string \"CA\" """
    postal_code: NotRequired[str]
    r"""Always the fixed string \"94107\" """
    phone_number: NotRequired[str]
    r"""Always the fixed string \"4157778888\" """


class ContractorPaymentReceiptLicensee(BaseModel):
    r"""The licensed payroll processor"""

    name: Optional[str] = None
    r"""Always the fixed string \"Gusto, Zenpayroll Inc.\" """

    address: Optional[str] = None
    r"""Always the fixed string \"525 20th St\" """

    city: Optional[str] = None
    r"""Always the fixed string \"San Francisco\" """

    state: Optional[str] = None
    r"""Always the fixed string \"CA\" """

    postal_code: Optional[str] = None
    r"""Always the fixed string \"94107\" """

    phone_number: Optional[str] = None
    r"""Always the fixed string \"4157778888\" """


class ContractorPaymentReceiptTypedDict(TypedDict):
    r"""Example response"""

    contractor_payment_uuid: NotRequired[str]
    r"""A unique identifier of the contractor payment receipt."""
    company_uuid: NotRequired[str]
    r"""A unique identifier of the company making the contractor payment."""
    name_of_sender: NotRequired[str]
    r"""The name of the company making the contractor payment."""
    name_of_recipient: NotRequired[str]
    r"""The individual or company name of the contractor receiving payment."""
    debit_date: NotRequired[date]
    r"""The debit date for the contractor payment."""
    license: NotRequired[str]
    r"""Always the fixed string \"Your payroll provider partners with Gusto Inc. for payments processing. Gusto Inc. is a licensed money transmitter. Learn more on our license page.\" """
    license_uri: NotRequired[str]
    r"""URL for the license information for the licensed payroll processor. Always the fixed string \"https://gusto.com/about/licenses\" """
    right_to_refund: NotRequired[str]
    r"""URL for information related to right to refund. Always the fixed string \"https://gusto.com/about/licenses\" """
    liability_of_licensee: NotRequired[str]
    r"""URL for information related to right to liability of licensee. Always the fixed string \"https://gusto.com/about/licenses\" """
    totals: NotRequired[ContractorPaymentReceiptTotalsTypedDict]
    r"""The subtotals for the contractor payment."""
    contractor_payments: NotRequired[List[ContractorPaymentsModelTypedDict]]
    r"""An array of contractor payments for this contractor payment."""
    licensee: NotRequired[ContractorPaymentReceiptLicenseeTypedDict]
    r"""The licensed payroll processor"""


class ContractorPaymentReceipt(BaseModel):
    r"""Example response"""

    contractor_payment_uuid: Optional[str] = None
    r"""A unique identifier of the contractor payment receipt."""

    company_uuid: Optional[str] = None
    r"""A unique identifier of the company making the contractor payment."""

    name_of_sender: Optional[str] = None
    r"""The name of the company making the contractor payment."""

    name_of_recipient: Optional[str] = None
    r"""The individual or company name of the contractor receiving payment."""

    debit_date: Optional[date] = None
    r"""The debit date for the contractor payment."""

    license: Optional[str] = None
    r"""Always the fixed string \"Your payroll provider partners with Gusto Inc. for payments processing. Gusto Inc. is a licensed money transmitter. Learn more on our license page.\" """

    license_uri: Optional[str] = None
    r"""URL for the license information for the licensed payroll processor. Always the fixed string \"https://gusto.com/about/licenses\" """

    right_to_refund: Optional[str] = None
    r"""URL for information related to right to refund. Always the fixed string \"https://gusto.com/about/licenses\" """

    liability_of_licensee: Optional[str] = None
    r"""URL for information related to right to liability of licensee. Always the fixed string \"https://gusto.com/about/licenses\" """

    totals: Optional[ContractorPaymentReceiptTotals] = None
    r"""The subtotals for the contractor payment."""

    contractor_payments: Optional[List[ContractorPaymentsModel]] = None
    r"""An array of contractor payments for this contractor payment."""

    licensee: Optional[ContractorPaymentReceiptLicensee] = None
    r"""The licensed payroll processor"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gusto_embedded import models, utils
from gusto_embedded._hooks import HookContext
from gusto_embedded.types import OptionalNullable, UNSET
from gusto_embedded.utils import get_security_from_env
from typing import Any, List, Mapping, Optional, Union


class EmployeeTaxSetup(BaseSDK):
    def get_federal_taxes(
        self,
        *,
        employee_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EmployeeFederalTax:
        r"""Get an employee's federal taxes

        Get attributes relevant for an employee's federal taxes.

        scope: `employee_federal_taxes:read`

        :param employee_uuid: The UUID of the employee
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeesEmployeeIDFederalTaxesRequest(
            employee_uuid=employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/employees/{employee_uuid}/federal_taxes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employees-employee_id-federal_taxes",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.EmployeeFederalTax)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_federal_taxes_async(
        self,
        *,
        employee_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EmployeeFederalTax:
        r"""Get an employee's federal taxes

        Get attributes relevant for an employee's federal taxes.

        scope: `employee_federal_taxes:read`

        :param employee_uuid: The UUID of the employee
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeesEmployeeIDFederalTaxesRequest(
            employee_uuid=employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/employees/{employee_uuid}/federal_taxes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employees-employee_id-federal_taxes",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.EmployeeFederalTax)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def update_federal_taxes(
        self,
        *,
        employee_uuid: str,
        version: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        filing_status: Optional[str] = None,
        extra_withholding: OptionalNullable[str] = UNSET,
        two_jobs: Optional[bool] = None,
        dependents_amount: Optional[str] = None,
        other_income: Optional[str] = None,
        deductions: Optional[str] = None,
        w4_data_type: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EmployeeFederalTax:
        r"""Update an employee's federal taxes

        Update attributes relevant for an employee's federal taxes.

        scope: `employee_federal_taxes:write`

        :param employee_uuid: The UUID of the employee
        :param version: The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field.
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param filing_status:
        :param extra_withholding:
        :param two_jobs:
        :param dependents_amount:
        :param other_income:
        :param deductions:
        :param w4_data_type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1EmployeesEmployeeIDFederalTaxesRequest(
            employee_uuid=employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1EmployeesEmployeeIDFederalTaxesRequestBody(
                version=version,
                filing_status=filing_status,
                extra_withholding=extra_withholding,
                two_jobs=two_jobs,
                dependents_amount=dependents_amount,
                other_income=other_income,
                deductions=deductions,
                w4_data_type=w4_data_type,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/employees/{employee_uuid}/federal_taxes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1EmployeesEmployeeIDFederalTaxesRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-employees-employee_id-federal_taxes",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.EmployeeFederalTax)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_federal_taxes_async(
        self,
        *,
        employee_uuid: str,
        version: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        filing_status: Optional[str] = None,
        extra_withholding: OptionalNullable[str] = UNSET,
        two_jobs: Optional[bool] = None,
        dependents_amount: Optional[str] = None,
        other_income: Optional[str] = None,
        deductions: Optional[str] = None,
        w4_data_type: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EmployeeFederalTax:
        r"""Update an employee's federal taxes

        Update attributes relevant for an employee's federal taxes.

        scope: `employee_federal_taxes:write`

        :param employee_uuid: The UUID of the employee
        :param version: The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field.
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param filing_status:
        :param extra_withholding:
        :param two_jobs:
        :param dependents_amount:
        :param other_income:
        :param deductions:
        :param w4_data_type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1EmployeesEmployeeIDFederalTaxesRequest(
            employee_uuid=employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1EmployeesEmployeeIDFederalTaxesRequestBody(
                version=version,
                filing_status=filing_status,
                extra_withholding=extra_withholding,
                two_jobs=two_jobs,
                dependents_amount=dependents_amount,
                other_income=other_income,
                deductions=deductions,
                w4_data_type=w4_data_type,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/employees/{employee_uuid}/federal_taxes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1EmployeesEmployeeIDFederalTaxesRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-employees-employee_id-federal_taxes",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.EmployeeFederalTax)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get_state_taxes(
        self,
        *,
        employee_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.EmployeeStateTax]:
        r"""Get an employee's state taxes

        Get attributes relevant for an employee's state taxes.

        The data required to correctly calculate an employee's state taxes varies by both home and work location. This API returns information about each question that must be answered grouped by state. Mostly commonly, an employee lives and works in the same state and will only have questions for a single state. The response contains metadata about each question, the type of answer expected, and the current answer stored in Gusto for that question.

        Answers are represented by an array. Today, this array can only be empty or contain exactly one element, but is designed to allow for forward compatibility with effective-dated fields. Until effective dated answers are supported, the `valid_from` and `valid_up_to` must always be `\"2010-01-01\"` and `null` respectively.

        ## About filing new hire reports
        Payroll Admins are responsible for filing a new hire report for each Employee. The `file_new_hire_report` question will only be listed if:
        - the `employee.onboarding_status` is one of the following:
        - `admin_onboarding_incomplete`
        - `self_onboarding_awaiting_admin_review`
        - that employee's work state requires filing a new hire report

        scope: `employee_state_taxes:read`


        :param employee_uuid: The UUID of the employee
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeesEmployeeIDStateTaxesRequest(
            employee_uuid=employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/employees/{employee_uuid}/state_taxes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employees-employee_id-state_taxes",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, List[models.EmployeeStateTax])
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_state_taxes_async(
        self,
        *,
        employee_uuid: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.EmployeeStateTax]:
        r"""Get an employee's state taxes

        Get attributes relevant for an employee's state taxes.

        The data required to correctly calculate an employee's state taxes varies by both home and work location. This API returns information about each question that must be answered grouped by state. Mostly commonly, an employee lives and works in the same state and will only have questions for a single state. The response contains metadata about each question, the type of answer expected, and the current answer stored in Gusto for that question.

        Answers are represented by an array. Today, this array can only be empty or contain exactly one element, but is designed to allow for forward compatibility with effective-dated fields. Until effective dated answers are supported, the `valid_from` and `valid_up_to` must always be `\"2010-01-01\"` and `null` respectively.

        ## About filing new hire reports
        Payroll Admins are responsible for filing a new hire report for each Employee. The `file_new_hire_report` question will only be listed if:
        - the `employee.onboarding_status` is one of the following:
        - `admin_onboarding_incomplete`
        - `self_onboarding_awaiting_admin_review`
        - that employee's work state requires filing a new hire report

        scope: `employee_state_taxes:read`


        :param employee_uuid: The UUID of the employee
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeesEmployeeIDStateTaxesRequest(
            employee_uuid=employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/employees/{employee_uuid}/state_taxes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employees-employee_id-state_taxes",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, List[models.EmployeeStateTax])
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def update_state_taxes(
        self,
        *,
        employee_uuid: str,
        states: Union[List[models.States], List[models.StatesTypedDict]],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.EmployeeStateTax]:
        r"""Update an employee's state taxes

        Update attributes relevant for an employee's state taxes.

        As described for the GET endpoint, the answers must be supplied in the effective-dated format, but currently only a single answer will be accepted - `valid_from` and `valid_up_to` must be `\"2010-01-01\"` and `null` respectively.

        scope: `employee_state_taxes:write`

        :param employee_uuid: The UUID of the employee
        :param states:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1EmployeesEmployeeIDStateTaxesRequest(
            employee_uuid=employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1EmployeesEmployeeIDStateTaxesRequestBody(
                states=utils.get_pydantic_model(states, List[models.States]),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/employees/{employee_uuid}/state_taxes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1EmployeesEmployeeIDStateTaxesRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-employees-employee_id-state_taxes",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, List[models.EmployeeStateTax])
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_state_taxes_async(
        self,
        *,
        employee_uuid: str,
        states: Union[List[models.States], List[models.StatesTypedDict]],
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.EmployeeStateTax]:
        r"""Update an employee's state taxes

        Update attributes relevant for an employee's state taxes.

        As described for the GET endpoint, the answers must be supplied in the effective-dated format, but currently only a single answer will be accepted - `valid_from` and `valid_up_to` must be `\"2010-01-01\"` and `null` respectively.

        scope: `employee_state_taxes:write`

        :param employee_uuid: The UUID of the employee
        :param states:
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1EmployeesEmployeeIDStateTaxesRequest(
            employee_uuid=employee_uuid,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1EmployeesEmployeeIDStateTaxesRequestBody(
                states=utils.get_pydantic_model(states, List[models.States]),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/employees/{employee_uuid}/state_taxes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1EmployeesEmployeeIDStateTaxesRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-employees-employee_id-state_taxes",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, List[models.EmployeeStateTax])
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DocumentType(str, Enum):
    PRINTABLE_PAYROLL_CHECKS = "printable_payroll_checks"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PayScheduleAssignmentTransitionPayPeriodTypedDict(TypedDict):
    r"""Pay schedule assignment transition pay period information."""

    start_date: NotRequired[str]
    r"""Pay period start date."""
    end_date: NotRequired[str]
    r"""Pay period end date."""


class PayScheduleAssignmentTransitionPayPeriod(BaseModel):
    r"""Pay schedule assignment transition pay period information."""

    start_date: Optional[str] = None
    r"""Pay period start date."""

    end_date: Optional[str] = None
    r"""Pay period end date."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    SecurityMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetV1WebhookSubscriptionVerificationTokenUUIDSecurityTypedDict(TypedDict):
    system_access_auth: str


class GetV1WebhookSubscriptionVerificationTokenUUIDSecurity(BaseModel):
    system_access_auth: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


class GetV1WebhookSubscriptionVerificationTokenUUIDRequestTypedDict(TypedDict):
    webhook_subscription_uuid: str
    r"""The webhook subscription UUID."""
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class GetV1WebhookSubscriptionVerificationTokenUUIDRequest(BaseModel):
    webhook_subscription_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The webhook subscription UUID."""

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .employee_state_tax_answer import (
    EmployeeStateTaxAnswer,
    EmployeeStateTaxAnswerTypedDict,
)
from .employee_state_tax_input_question_format import (
    EmployeeStateTaxInputQuestionFormat,
    EmployeeStateTaxInputQuestionFormatTypedDict,
)
from gusto_embedded.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class EmployeeStateTaxQuestionTypedDict(TypedDict):
    label: str
    r"""A short title for the question"""
    description: str
    r"""An explaination of the question - this may contain inline html formatted links."""
    key: str
    r"""A unique identifier of the question (for the given state) - used for updating the answer."""
    input_question_format: EmployeeStateTaxInputQuestionFormatTypedDict
    answers: List[EmployeeStateTaxAnswerTypedDict]


class EmployeeStateTaxQuestion(BaseModel):
    label: str
    r"""A short title for the question"""

    description: str
    r"""An explaination of the question - this may contain inline html formatted links."""

    key: str
    r"""A unique identifier of the question (for the given state) - used for updating the answer."""

    input_question_format: EmployeeStateTaxInputQuestionFormat

    answers: List[EmployeeStateTaxAnswer]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class PayrollEmployeeCompensationsTypePaymentMethod(str, Enum):
    r"""The employee's compensation payment method."""

    CHECK = "Check"
    DIRECT_DEPOSIT = "Direct Deposit"


class FixedCompensationsTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the compensation. This also serves as the unique, immutable identifier for this compensation."""
    amount: NotRequired[str]
    r"""The amount of the compensation for the pay period."""
    job_uuid: NotRequired[str]
    r"""The UUID of the job for the compensation."""


class FixedCompensations(BaseModel):
    name: Optional[str] = None
    r"""The name of the compensation. This also serves as the unique, immutable identifier for this compensation."""

    amount: Optional[str] = None
    r"""The amount of the compensation for the pay period."""

    job_uuid: Optional[str] = None
    r"""The UUID of the job for the compensation."""


class HourlyCompensationsTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the compensation. This also serves as the unique, immutable identifier for this compensation."""
    hours: NotRequired[str]
    r"""The number of hours to be compensated for this pay period."""
    amount: NotRequired[str]
    r"""The amount of the compensation. This field is only available after the payroll is calculated and cannot be used for updating hourly compensations."""
    job_uuid: NotRequired[str]
    r"""The UUID of the job for the compensation."""
    compensation_multiplier: NotRequired[float]
    r"""The amount multiplied by the base rate to calculate total compensation per hour worked."""
    flsa_status: NotRequired[str]
    r"""The FLSA Status of the employee's primary job compensation"""


class HourlyCompensations(BaseModel):
    name: Optional[str] = None
    r"""The name of the compensation. This also serves as the unique, immutable identifier for this compensation."""

    hours: Optional[str] = None
    r"""The number of hours to be compensated for this pay period."""

    amount: Optional[str] = None
    r"""The amount of the compensation. This field is only available after the payroll is calculated and cannot be used for updating hourly compensations."""

    job_uuid: Optional[str] = None
    r"""The UUID of the job for the compensation."""

    compensation_multiplier: Optional[float] = None
    r"""The amount multiplied by the base rate to calculate total compensation per hour worked."""

    flsa_status: Optional[str] = None
    r"""The FLSA Status of the employee's primary job compensation"""


class PayrollEmployeeCompensationsTypePaidTimeOffTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the PTO. This also serves as the unique, immutable identifier for the PTO."""
    hours: NotRequired[str]
    r"""The hours of this PTO taken during the pay period."""
    final_payout_unused_hours_input: NotRequired[str]
    r"""The outstanding hours paid upon termination. This field is only applicable for termination payrolls."""


class PayrollEmployeeCompensationsTypePaidTimeOff(BaseModel):
    name: Optional[str] = None
    r"""The name of the PTO. This also serves as the unique, immutable identifier for the PTO."""

    hours: Optional[str] = None
    r"""The hours of this PTO taken during the pay period."""

    final_payout_unused_hours_input: Optional[str] = None
    r"""The outstanding hours paid upon termination. This field is only applicable for termination payrolls."""


class BenefitsTypedDict(TypedDict):
    name: NotRequired[str]
    employee_deduction: NotRequired[float]
    company_contribution: NotRequired[float]
    imputed: NotRequired[bool]


class Benefits(BaseModel):
    name: Optional[str] = None

    employee_deduction: Optional[float] = None

    company_contribution: Optional[float] = None

    imputed: Optional[bool] = None


class DeductionsTypedDict(TypedDict):
    name: NotRequired[str]
    amount: NotRequired[float]


class Deductions(BaseModel):
    name: Optional[str] = None

    amount: Optional[float] = None


class TaxesTypedDict(TypedDict):
    name: str
    employer: bool
    amount: float


class Taxes(BaseModel):
    name: str

    employer: bool

    amount: float


class PayrollEmployeeCompensationsTypeTypedDict(TypedDict):
    employee_uuid: NotRequired[str]
    r"""The UUID of the employee."""
    excluded: NotRequired[bool]
    r"""This employee will be excluded (skipped) from payroll calculation and will not be paid for the payroll. Cancelling a payroll would reset all employees' excluded back to false."""
    version: NotRequired[str]
    r"""The current version of this employee compensation. This field is only available for prepared payrolls. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""
    gross_pay: NotRequired[Nullable[float]]
    r"""The employee's gross pay, equal to regular wages + cash tips + payroll tips + any other additional earnings, excluding imputed income. This value is only available for processed payrolls."""
    net_pay: NotRequired[Nullable[float]]
    r"""The employee's net pay, equal to gross_pay - employee taxes - employee deductions or garnishments - cash tips. This value is only available for processed payrolls."""
    check_amount: NotRequired[Nullable[float]]
    r"""The employee's check amount, equal to net_pay + reimbursements. This value is only available for processed payrolls."""
    payment_method: NotRequired[Nullable[PayrollEmployeeCompensationsTypePaymentMethod]]
    r"""The employee's compensation payment method."""
    memo: NotRequired[Nullable[str]]
    r"""Custom text that will be printed as a personal note to the employee on a paystub."""
    fixed_compensations: NotRequired[List[FixedCompensationsTypedDict]]
    r"""An array of fixed compensations for the employee. Fixed compensations include tips, bonuses, and one time reimbursements. If this payroll has been processed, only fixed compensations with a value greater than 0.00 are returned. For an unprocessed payroll, all active fixed compensations are returned."""
    hourly_compensations: NotRequired[List[HourlyCompensationsTypedDict]]
    r"""An array of hourly compensations for the employee. Hourly compensations include regular, overtime, and double overtime hours. If this payroll has been processed, only hourly compensations with a value greater than 0.00 are returned. For an unprocessed payroll, all active hourly compensations are returned."""
    paid_time_off: NotRequired[
        List[PayrollEmployeeCompensationsTypePaidTimeOffTypedDict]
    ]
    r"""An array of all paid time off the employee is eligible for this pay period."""
    benefits: NotRequired[List[BenefitsTypedDict]]
    r"""An array of employee benefits for the pay period. Benefits are only included for processed payroll when the include parameter is present."""
    deductions: NotRequired[List[DeductionsTypedDict]]
    r"""An array of employee deductions for the pay period. Deductions are only included for processed payroll when the include parameter is present."""
    taxes: NotRequired[List[TaxesTypedDict]]
    r"""An array of employer and employee taxes for the pay period. Only included for processed or calculated payrolls when `taxes` is present in the `include` parameter."""


class PayrollEmployeeCompensationsType(BaseModel):
    employee_uuid: Optional[str] = None
    r"""The UUID of the employee."""

    excluded: Optional[bool] = None
    r"""This employee will be excluded (skipped) from payroll calculation and will not be paid for the payroll. Cancelling a payroll would reset all employees' excluded back to false."""

    version: Optional[str] = None
    r"""The current version of this employee compensation. This field is only available for prepared payrolls. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""

    gross_pay: OptionalNullable[float] = UNSET
    r"""The employee's gross pay, equal to regular wages + cash tips + payroll tips + any other additional earnings, excluding imputed income. This value is only available for processed payrolls."""

    net_pay: OptionalNullable[float] = UNSET
    r"""The employee's net pay, equal to gross_pay - employee taxes - employee deductions or garnishments - cash tips. This value is only available for processed payrolls."""

    check_amount: OptionalNullable[float] = UNSET
    r"""The employee's check amount, equal to net_pay + reimbursements. This value is only available for processed payrolls."""

    payment_method: OptionalNullable[PayrollEmployeeCompensationsTypePaymentMethod] = (
        UNSET
    )
    r"""The employee's compensation payment method."""

    memo: OptionalNullable[str] = UNSET
    r"""Custom text that will be printed as a personal note to the employee on a paystub."""

    fixed_compensations: Optional[List[FixedCompensations]] = None
    r"""An array of fixed compensations for the employee. Fixed compensations include tips, bonuses, and one time reimbursements. If this payroll has been processed, only fixed compensations with a value greater than 0.00 are returned. For an unprocessed payroll, all active fixed compensations are returned."""

    hourly_compensations: Optional[List[HourlyCompensations]] = None
    r"""An array of hourly compensations for the employee. Hourly compensations include regular, overtime, and double overtime hours. If this payroll has been processed, only hourly compensations with a value greater than 0.00 are returned. For an unprocessed payroll, all active hourly compensations are returned."""

    paid_time_off: Optional[List[PayrollEmployeeCompensationsTypePaidTimeOff]] = None
    r"""An array of all paid time off the employee is eligible for this pay period."""

    benefits: Optional[List[Benefits]] = None
    r"""An array of employee benefits for the pay period. Benefits are only included for processed payroll when the include parameter is present."""

    deductions: Optional[List[Deductions]] = None
    r"""An array of employee deductions for the pay period. Deductions are only included for processed payroll when the include parameter is present."""

    taxes: Optional[List[Taxes]] = None
    r"""An array of employer and employee taxes for the pay period. Only included for processed or calculated payrolls when `taxes` is present in the `include` parameter."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "employee_uuid",
            "excluded",
            "version",
            "gross_pay",
            "net_pay",
            "check_amount",
            "payment_method",
            "memo",
            "fixed_compensations",
            "hourly_compensations",
            "paid_time_off",
            "benefits",
            "deductions",
            "taxes",
        ]
        nullable_fields = [
            "gross_pay",
            "net_pay",
            "check_amount",
            "payment_method",
            "memo",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutV1CompaniesCompanyUUIDTaxRequirementsStateRequirementsTypedDict(TypedDict):
    key: NotRequired[str]
    r"""An identifier for an individual requirement. Uniqueness is guaranteed within a requirement set."""
    value: NotRequired[Nullable[str]]


class PutV1CompaniesCompanyUUIDTaxRequirementsStateRequirements(BaseModel):
    key: Optional[str] = None
    r"""An identifier for an individual requirement. Uniqueness is guaranteed within a requirement set."""

    value: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["key", "value"]
        nullable_fields = ["value"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class RequirementSetsTypedDict(TypedDict):
    key: NotRequired[str]
    r"""An identifier for a set of requirements. A list of requirement sets can contain multiple sets with the same `key` and different `effective_from` values."""
    effective_from: NotRequired[Nullable[str]]
    r"""An ISO 8601 formatted date representing the date values became effective. Some requirement sets are effective dated, while others are not. Multiple requirement sets for the same state/key can/will exist with unique effective dates. If a requirement set is has an `effective_from` value, all requirement sets with the same key will also have an `effective_from` value."""
    state: NotRequired[str]
    r"""One of the two-letter state abbreviations for the fifty United States and the District of Columbia (DC)"""
    requirements: NotRequired[
        List[PutV1CompaniesCompanyUUIDTaxRequirementsStateRequirementsTypedDict]
    ]


class RequirementSets(BaseModel):
    key: Optional[str] = None
    r"""An identifier for a set of requirements. A list of requirement sets can contain multiple sets with the same `key` and different `effective_from` values."""

    effective_from: OptionalNullable[str] = UNSET
    r"""An ISO 8601 formatted date representing the date values became effective. Some requirement sets are effective dated, while others are not. Multiple requirement sets for the same state/key can/will exist with unique effective dates. If a requirement set is has an `effective_from` value, all requirement sets with the same key will also have an `effective_from` value."""

    state: Optional[str] = None
    r"""One of the two-letter state abbreviations for the fifty United States and the District of Columbia (DC)"""

    requirements: Optional[
        List[PutV1CompaniesCompanyUUIDTaxRequirementsStateRequirements]
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["key", "effective_from", "state", "requirements"]
        nullable_fields = ["effective_from"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PutV1CompaniesCompanyUUIDTaxRequirementsStateRequestBodyTypedDict(TypedDict):
    requirement_sets: NotRequired[List[RequirementSetsTypedDict]]


class PutV1CompaniesCompanyUUIDTaxRequirementsStateRequestBody(BaseModel):
    requirement_sets: Optional[List[RequirementSets]] = None


class PutV1CompaniesCompanyUUIDTaxRequirementsStateRequestTypedDict(TypedDict):
    company_uuid: str
    r"""The UUID of the company"""
    state: str
    r"""2-letter US state abbreviation"""
    request_body: PutV1CompaniesCompanyUUIDTaxRequirementsStateRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1CompaniesCompanyUUIDTaxRequirementsStateRequest(BaseModel):
    company_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    state: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""2-letter US state abbreviation"""

    request_body: Annotated[
        PutV1CompaniesCompanyUUIDTaxRequirementsStateRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PostV1CompaniesCompanyIDLocationsRequestBodyTypedDict(TypedDict):
    r"""Create a company location."""

    phone_number: str
    street_1: str
    city: str
    state: str
    zip_code: str
    street_2: NotRequired[Nullable[str]]
    mailing_address: NotRequired[bool]
    r"""Specify if this location is the company's mailing address."""
    filing_address: NotRequired[bool]
    r"""Specify if this location is the company's filing address."""


class PostV1CompaniesCompanyIDLocationsRequestBody(BaseModel):
    r"""Create a company location."""

    phone_number: str

    street_1: str

    city: str

    state: str

    zip_code: Annotated[str, pydantic.Field(alias="zip")]

    street_2: OptionalNullable[str] = UNSET

    mailing_address: Optional[bool] = None
    r"""Specify if this location is the company's mailing address."""

    filing_address: Optional[bool] = None
    r"""Specify if this location is the company's filing address."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["street_2", "mailing_address", "filing_address"]
        nullable_fields = ["street_2"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PostV1CompaniesCompanyIDLocationsRequestTypedDict(TypedDict):
    company_id: str
    r"""The UUID of the company"""
    request_body: PostV1CompaniesCompanyIDLocationsRequestBodyTypedDict
    r"""Create a company location."""
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PostV1CompaniesCompanyIDLocationsRequest(BaseModel):
    company_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    request_body: Annotated[
        PostV1CompaniesCompanyIDLocationsRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Create a company location."""

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""
